;============================================
; Resizing Test Bed
;
; Tests resizing objects (child windows)
; using owner-drawn resizing handles
;
; LESSONS LEARNED IN THIS CODE:
;-------------------------------
; o Loading cursors, setting in child windows
; o Moving multiple child windows as a unit
; o Using pop-up ("shortcut") menus
; o Creating child-window controls "on the fly"
; o Mouse capture, window moving and coordination
;   between parent & child window procs
;
; UNSOLVED PROBLEMS:
;--------------------
; o Static text doesn't show in child window
;============================================


.386
.model flat, stdcall
option casemap :none

.nolist
	include \masm32\include\windows.inc
	include \masm32\include\user32.inc
	include \masm32\include\kernel32.inc
	include \masm32\include\gdi32.inc
	include \masm32\include\comctl32.inc

	includelib \masm32\lib\gdi32.lib
	includelib \masm32\lib\user32.lib
	includelib \masm32\lib\kernel32.lib
	includelib \masm32\lib\Comctl32.lib
	includelib \masm32\lib\comdlg32.lib
	includelib \masm32\lib\shell32.lib
	includelib \masm32\lib\oleaut32.lib
	includelib \masm32\lib\ole32.lib
	includelib \masm32\lib\msvcrt.lib
.list


;============================================
; Defines, macros, prototypes, etc.
;============================================

WinMain			PROTO :DWORD
MoveHandleWindows	PROTO x0c:DWORD, y0c:DWORD

$handleInfo		STRUCT
  ID			DD ?
  handle		HWND ?
  X			DD ?
  Y			DD ?
  cursor		HCURSOR ?
$handleInfo		ENDS

$mainWindowWidth	EQU 700
$mainWindowHeight	EQU 300

$childX			EQU 100
$childY			EQU 100
$Wc			EQU 200
$Hc			EQU 40

$Wh			EQU 6
$Hh			EQU 6

$Xc			EQU $childX
$Yc			EQU $childY

$chkID			EQU 2020
$chkX			EQU 20
$chkY			EQU 10
$chkWidth		EQU 50
$chkHeight		EQU 20

$radioID		EQU 2030
$radioX			EQU 20
$radioY			EQU 10
$radioWidth		EQU 50
$radioHeight		EQU 20

$editID			EQU 2031
$editX			EQU 20
$editY			EQU 10
$editWidth		EQU 50
$editHeight		EQU 20

$STID			EQU 2032
$STX			EQU 20
$STY			EQU 10
$STWidth		EQU 80
$STHeight		EQU 20

;===== Window styles: =====
$mainWinAttrs		EQU WS_OVERLAPPEDWINDOW OR WS_CLIPCHILDREN
$childWinAttrs		EQU WS_CHILD OR WS_VISIBLE OR WS_CLIPSIBLINGS
$handleWinAttrs		EQU WS_CHILD OR WS_BORDER OR WS_VISIBLE OR WS_CLIPSIBLINGS

;===== Window background colors: =====
$bkRED			EQU 254
$bkGRN			EQU 243
$bkBLUE			EQU 199

$whtRED			EQU 255
$whtGRN			EQU 255
$whtBLUE		EQU 255

$BkColor		EQU $bkRED OR ($bkGRN SHL 8) OR ($bkBLUE SHL 16)
$white			EQU $whtRED OR ($whtGRN SHL 8) OR ($whtBLUE SHL 16)

;============================================
; DATA
;============================================
.data

InstanceHandle		HWND ?
MainWinHandle		HWND ?
ChildWinHandle		HWND ?
ChkHandle		HWND ?
RadioHandle		HWND ?
EditHandle		HWND ?
STHandle		HWND ?

ControlHandle		HWND NULL

ShortcutMenuHandle	HMENU ?

HandleInfo	LABEL DWORD
			$handleInfo <2000, 0, $Xc - $Wh, $Yc - $Hh>			;1
			$handleInfo <2001, 0, $Xc + ($Wc / 2) + ($Wh / 2), $Yc - $Hh>	;2
			$handleInfo <2002, 0, $Xc + $Wc, $Yc - $Hh>			;3
			$handleInfo <2003, 0, $Xc - $Wh, $Yc + ($Hc / 2) - ($Hh / 2)>	;4
			$handleInfo <2004, 0, $Xc + $Wc, $Yc + ($Hc / 2) - ($Hh / 2)>	;5
			$handleInfo <2005, 0, $Xc - $Wh, $Yc + $Hc>			;6
			$handleInfo <2006, 0, $Xc + ($Wc / 2) + ($Wh / 2), $Yc + $Hc>	;7
			$handleInfo <2007, 0, $Xc + $Wc, $Yc + $Hc>			;8

MM_Xoffset		DD ?
MM_Yoffset		DD ?

PrevRect		RECT <>

;===== Byte-size items: =====
MovingChild		DB FALSE
ClickInChild		DB FALSE

MainClassName		DB "Resizing", 0
ChildClassName		DB "static", 0
BtnClassName		DB "button", 0
EditClassName		DB "edit", 0
StaticClassName		DB "static", 0

MainDisplayName		DB "Resizing Handle Test", 0
ChildText		DB "Child", 0
HandleText		DB "H", 0
ChkText			DB "Fur:", 0
RadioText		DB "FM", 0
StaticText		DB "Some text", 0

Menu_CheckboxText	DB "Checkbox", 0
Menu_RadioText		DB "Radio button", 0
Menu_EditText		DB "Edit box", 0
Menu_StaticText		DB "Static text", 0

heremsg			DB "Here!", 0
registerclassfailmsg	DB "RegisterClassEx() FAILED!", 0
msg2223			DB "You selected Radio button", 0


;============================================
; CODE
;============================================
.code


start:
	INVOKE	GetModuleHandle, NULL
	MOV	InstanceHandle, EAX

	INVOKE	WinMain, InstanceHandle
	INVOKE	ExitProcess, EAX


;====================================================================
; Mainline proc
;====================================================================

WinMain	PROC 	hInst:DWORD
	LOCAL	wc:WNDCLASSEX, msg:MSG, brush:HBRUSH, wX:DWORD, wY:DWORD


; Create  brush to set background color:
	INVOKE	CreateSolidBrush, $BkColor
	MOV	brush, EAX

; Register class for parent window:
	MOV	wc.cbSize, SIZEOF WNDCLASSEX
	MOV	wc.style,  CS_HREDRAW or CS_VREDRAW or CS_BYTEALIGNWINDOW
	MOV	wc.lpfnWndProc, OFFSET MainWindowProc
	MOV	wc.cbClsExtra, NULL
	MOV	wc.cbWndExtra, NULL
	MOV	EAX, hInst
	MOV	wc.hInstance, EAX
	MOV	EAX, brush
	MOV	wc.hbrBackground, EAX
	MOV	wc.lpszMenuName,  NULL
	MOV	wc.lpszClassName, OFFSET MainClassName
	MOV	wc.hIcon, NULL
	INVOKE	LoadCursor, NULL, IDC_ARROW
	MOV	wc.hCursor, EAX
	MOV	wc.hIconSm, 0
	INVOKE	RegisterClassEx, ADDR wc

; Register class for child windows:
	MOV	wc.cbSize, SIZEOF WNDCLASSEX
	MOV	wc.style,  CS_HREDRAW or CS_VREDRAW or CS_BYTEALIGNWINDOW
	MOV	wc.lpfnWndProc, OFFSET ChildWindowProc
	MOV	wc.cbClsExtra, 0
	MOV	wc.cbWndExtra, 0
	MOV	EAX, hInst
	MOV	wc.hInstance, EAX
	MOV	wc.hbrBackground, COLOR_BTNFACE + 1
	MOV	wc.lpszMenuName,  NULL
	MOV	wc.lpszClassName, OFFSET ChildClassName
	MOV	wc.hIcon, NULL
	INVOKE	LoadCursor, NULL, IDC_ARROW
	MOV	wc.hCursor, EAX
	MOV	wc.hIconSm, 0
	INVOKE	RegisterClassEx, ADDR wc

	INVOKE	GetSystemMetrics, SM_CXSCREEN
	MOV	EDX, $mainWindowWidth
	CALL	CenterDim
	MOV	wX, EAX
	INVOKE	GetSystemMetrics, SM_CYSCREEN
	MOV	EDX, $mainWindowHeight
	CALL	CenterDim
	MOV	wY, EAX

; Create our main window:
	INVOKE	CreateWindowEx, WS_EX_OVERLAPPEDWINDOW, ADDR MainClassName, ADDR MainDisplayName,
		$mainWinAttrs, wX, wY, $mainWindowWidth, $mainWindowHeight,
		NULL, NULL, hInst, NULL
	MOV	MainWinHandle, EAX
	INVOKE	ShowWindow, EAX, SW_SHOWNORMAL

; Load resizing-handle cursors:
	CALL	LoadArrowCursors

; Create child window:
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR ChildClassName, NULL,
		$childWinAttrs, $childX, $childY, $Wc, $Hc,
		MainWinHandle, 1999, hInst, NULL
	MOV	ChildWinHandle, EAX
	INVOKE	UpdateWindow, EAX
;	INVOKE	ShowWindow, EAX, SW_SHOWNORMAL

; Create handle windows:
	PUSH	EBX
	LEA	EBX, HandleInfo				;Point to handle info structure array
	MOV	ECX, 8
chw10:	PUSH	ECX
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR ChildClassName, NULL, $handleWinAttrs,
     		[EBX + $handleInfo.X], [EBX + $handleInfo.Y], $Wh, $Hh,
		MainWinHandle, [EBX + $handleInfo.ID], InstanceHandle, 0
	MOV	[EBX + $handleInfo.handle], EAX		;Store handle.
	INVOKE	UpdateWindow, EAX			;Force child wndproc to paint window.
	ADD	EBX, SIZEOF $handleInfo
	POP	ECX
	LOOP	chw10
chw99:	POP	EBX



;============= Message loop ===================
msgloop:
	INVOKE	GetMessage, ADDR msg, NULL, 0, 0
	OR	EAX, EAX				;EAX = 0 = exit
	JZ	exit
	INVOKE	TranslateMessage, ADDR msg
	INVOKE	DispatchMessage, ADDR msg
	JMP	msgloop

exit:	MOV	EAX, msg.wParam
	RET

WinMain		ENDP


;====================================================================
; Main Window Proc
;====================================================================

MainWindowProc	PROC hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
	LOCAL	pt:POINT, x0c:DWORD, y0c:DWORD, gpRect:RECT, hMenu:HMENU

	MOV	EAX, uMsg
	CMP	EAX, WM_MOUSEMOVE
	JE	domousemove
	CMP	EAX, WM_LBUTTONUP
	JE	lbuttonup
	CMP	EAX, WM_RBUTTONDOWN
	JE	rbuttondown
	CMP	EAX, WM_COMMAND
	JE	dowincmd
;	CMP	EAX, WM_CREATE
;	JE	docreate
	CMP	EAX, WM_CLOSE
	JE	doclose

dodefault:
; use DefWindowProc for all other messages:
	INVOKE	DefWindowProc, hWin, uMsg, wParam, lParam
	RET

domousecapture:
	MOV	ClickInChild, FALSE
	MOV	MovingChild, TRUE
	INVOKE	SetCapture, hWin
	INVOKE	GetClipCursor, ADDR PrevRect	;Get current clip rectangle
	INVOKE	GetWindowRect, hWin, ADDR gpRect
 	INVOKE	ClipCursor, ADDR gpRect
	JMP	SHORT move20			;Start moving children

domousemove:
	CMP	ClickInChild, TRUE		;Supposed to watch the mouse?
	JE	domousecapture
	CMP	MovingChild, TRUE		;Are we watching the mouse?
	JNE	dodefault			;No, ignore message.

; First, get current mouse position (from lParam):
move20:	MOV	EAX, lParam			;Get new X & Y pos.
	MOV	EDX, EAX			;Make a copy for later
	AND	EAX, 0FFFFH			;Mask off high word
	CWDE					;Sign-extend to DWORD
	MOV	pt.x, EAX
	MOV	EAX, EDX			;Get back copy
	SHR	EAX, 16				;High word--> low word
	CWDE
	MOV	pt.y, EAX

;Move child window:
	MOV	EAX, pt.x
	SUB	EAX, MM_Xoffset
	MOV	x0c, EAX			;Compute new (0,0) of child window pos:
	MOV	EDX, pt.y
	SUB	EDX, MM_Yoffset
	MOV	y0c, EDX
	INVOKE	SetWindowPos, ChildWinHandle, HWND_TOP, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, ChildWinHandle, NULL, TRUE
	INVOKE	MoveHandleWindows, x0c, y0c
	JMP	mwp_commonexit


lbuttonup:
	CMP	MovingChild, TRUE		;Are we watching the mouse?
	JNE	dodefault			;Nope.
	MOV	MovingChild, FALSE		;Yep, clear flag.
	INVOKE	ReleaseCapture
	JMP	mwp_commonexit

rbuttondown:
	CALL	CreatePopupMenu			;Create empty shortcut menu
	MOV	hMenu, EAX 
	INVOKE	AppendMenu, hMenu, MF_STRING, 2222, OFFSET Menu_CheckboxText
	INVOKE	AppendMenu, hMenu, MF_STRING, 2223, OFFSET Menu_RadioText
	INVOKE	AppendMenu, hMenu, MF_STRING, 2225, OFFSET Menu_StaticText
	INVOKE	AppendMenu, hMenu, MF_STRING, 2224, OFFSET Menu_EditText
	INVOKE	ClientToScreen, hWin, ADDR pt	;Where is our client in the "real world"?
	MOV	EAX, lParam
	MOV	EDX, EAX
	AND	EAX, 0FFFFH			;Low word = X
	ADD	EAX, pt.x
	SHR	EDX, 16				;High word = Y
	ADD	EDX, pt.y
	INVOKE	TrackPopupMenuEx, hMenu, 
		TPM_LEFTALIGN OR TPM_TOPALIGN OR TPM_RETURNCMD OR TPM_RIGHTBUTTON OR TPM_VERNEGANIMATION OR TPM_VERTICAL,
		EAX, EDX, hWin, NULL

	CMP	EAX, 2222
	JE	makeCheckbox
	CMP	EAX, 2223
	JE	makeRadio
	CMP	EAX, 2224
	JE	makeEdit
	CMP	EAX, 2225
	JE	makeStaticText

	XOR	EAX, EAX
	RET

dowincmd:

docreate:
mwp_commonexit:
	XOR	EAX, EAX
	RET

doclose:
	INVOKE	PostQuitMessage, NULL
	MOV	EAX, TRUE
	RET

makeCheckbox:
	CALL	DestroyExistingControl
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR BtnClassName, ADDR ChkText,
		WS_TABSTOP OR WS_VISIBLE OR WS_CHILD OR BS_AUTOCHECKBOX, $chkX, $chkY,
		$chkWidth, $chkHeight, ChildWinHandle, $chkID, InstanceHandle, NULL
	MOV	ChkHandle, EAX
	MOV	ControlHandle, EAX
	INVOKE	CheckDlgButton, ChildWinHandle, $chkID, BST_CHECKED	;Set button state to checked.
	INVOKE	UpdateWindow, ChkHandle
	JMP	mwp_commonexit

makeRadio:
	CALL	DestroyExistingControl
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR BtnClassName, ADDR RadioText,
		WS_TABSTOP OR WS_VISIBLE OR WS_CHILD OR BS_AUTORADIOBUTTON, $radioX, $radioY,
		$radioWidth, $radioHeight, ChildWinHandle, $radioID, InstanceHandle, NULL
	MOV	RadioHandle, EAX
	MOV	ControlHandle, EAX
	INVOKE	CheckDlgButton, ChildWinHandle, $radioID, BST_CHECKED	;Set button state to checked.
	INVOKE	UpdateWindow, RadioHandle
	JMP	mwp_commonexit

makeEdit:
	CALL	DestroyExistingControl
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR EditClassName, NULL,
		SS_LEFT OR WS_VISIBLE OR WS_CHILD OR WS_BORDER, $editX, $editY,
		$editWidth, $editHeight, ChildWinHandle, $editID, InstanceHandle, NULL
	MOV	EditHandle, EAX
	MOV	ControlHandle, EAX
	INVOKE	UpdateWindow, EAX
	JMP	mwp_commonexit

makeStaticText:
	CALL	DestroyExistingControl
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR StaticClassName, ADDR StaticText,
		SS_LEFT OR WS_VISIBLE OR WS_CHILD, $STX, $STY,
		$STWidth, $STHeight, ChildWinHandle, $STID, InstanceHandle, NULL
	MOV	STHandle, EAX
	MOV	ControlHandle, EAX
;	INVOKE	InvalidateRect, EAX, NULL, FALSE
	INVOKE	UpdateWindow, EAX
	JMP	mwp_commonexit

MainWindowProc	ENDP


;====================================================================
; Child Window Proc
;====================================================================

ChildWindowProc	PROC hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
	LOCAL	gpRect:RECT

	MOV	EAX, uMsg
	CMP	EAX, WM_SETCURSOR
	JE	setcursor
	CMP	EAX, WM_LBUTTONDOWN
	JE	lbuttondown
	CMP	EAX, WM_LBUTTONUP
	JE	lbuttonup

dodefault:
; use DefWindowProc for all other messages:
	INVOKE	DefWindowProc, hWin, uMsg, wParam, lParam
	RET

setcursor:
	MOV	EAX, hWin
	CALL	FindHandleWindow		;See if handle matches any handle window's:
	OR	EAX, EAX			;Any match?
	JZ	dodefault			;Nope, do nothing.
	INVOKE	SetCursor, EAX
	XOR	EAX, EAX
	RET

lbuttondown:
; Get current mouse position, save as offset from (0,0):
	CMP	MovingChild, TRUE		;If already moving this window,
	JE	dodefault			; skip.
	MOV	EAX, lParam			;Get new X & Y pos.
	MOV	EDX, EAX			;Make a copy for later
	AND	EAX, 0FFFFH			;Mask off high word
	CWDE					;Sign-extend to DWORD
	MOV	MM_Xoffset, EAX
	MOV	EAX, EDX			;Get back copy
	SHR	EAX, 16				;High word--> low word
	CWDE
	MOV	MM_Yoffset, EAX

	MOV	ClickInChild, TRUE		;Indicate user clicked here
	JMP	SHORT cwp_commonexit

lbuttonup:
	CMP	MovingChild, TRUE		;Are we watching the mouse?
	JNE	dodefault			;Nope.
	MOV	MovingChild, FALSE		;Yep, clear flag.
	INVOKE	ReleaseCapture

cwp_commonexit:
	XOR	EAX, EAX
	RET

ChildWindowProc	ENDP


DestroyExistingControl	PROC

	CMP	ControlHandle, NULL		;Has a control been created?
	JE	dec99				;No, do nothing.
	INVOKE	DestroyWindow, ControlHandle
	MOV	ControlHandle, NULL
dec99:	RET

DestroyExistingControl	ENDP


;====================================================================
; MoveHandleWindows()
;
; Move sizing-handle windows relative to new (0,0) of the child
; window they surround.
;====================================================================

MoveHandleWindows	PROC x0c:DWORD, y0c:DWORD

	PUSH	EBX
	LEA	EBX, HandleInfo
; 1:
	MOV	EAX, x0c
	SUB	EAX, $Wh
	MOV	EDX, y0c
	SUB	EDX, $Hh
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_TOP, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 2: X = x0c + ($Wc / 2) + ($Wh / 2); Y = y0c - $Hh
	MOV	EAX, ($Wc SHR 1) + ($Wh SHR 1)
	ADD	EAX, x0c
	MOV	EDX, y0c
	SUB	EDX, $Hh
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_TOP, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 3: X = x0c + $Wc; Y = y0c - $Hh
	MOV	EAX, x0c
	ADD	EAX, $Wc
	MOV	EDX, y0c
	SUB	EDX, $Hh
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_TOP, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 4: X = x0c - $Wh; Y = y0c + ($Hc / 2) - ($Hh / 2)
	MOV	EAX, x0c
	SUB	EAX, $Wh
	MOV	EDX, ($Hc SHR 1) - ($Hh SHR 1)
	ADD	EDX, y0c
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_TOP, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 5: X = x0c + $Wc; Y = y0c + ($Hc / 2) - ($Hh / 2)
	MOV	EAX, x0c
	ADD	EAX, $Wc
	MOV	EDX, ($Hc SHR 1) - ($Hh SHR 1)
	ADD	EDX, y0c
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_TOP, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 6: X = x0c - $Wh; Y = y0c + $Hc
	MOV	EAX, x0c
	SUB	EAX, $Wh
	MOV	EDX, y0c
	ADD	EDX, $Hc
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_TOP, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 7: X = x0c + ($Wc / 2) + ($Wh / 2); Y = y0c + $Hc
	MOV	EAX, x0c
	ADD	EAX, ($Wc SHR 1)
	MOV	EDX, y0c
	ADD	EDX, $Hc
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_TOP, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 8: X = x0c + $Wc; Y = y0c + $Hc
	MOV	EAX, x0c
	ADD	EAX, $Wc
	MOV	EDX, y0c
	ADD	EDX, $Hc
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_TOP, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	
	POP	EBX
	RET

MoveHandleWindows	ENDP


LoadArrowCursors	PROC
	LOCAL	cursorNS:HCURSOR, cursorEW:HCURSOR, cursorLdiag:HCURSOR, cursorRdiag:HCURSOR

; Load 4 resizing cursors:
	INVOKE	LoadCursor, NULL, IDC_SIZENS
	MOV	cursorNS, EAX
	INVOKE	LoadCursor, NULL, IDC_SIZEWE
	MOV	cursorEW, EAX
	INVOKE	LoadCursor, NULL, IDC_SIZENWSE
	MOV	cursorLdiag, EAX
	INVOKE	LoadCursor, NULL, IDC_SIZENESW
	MOV	cursorRdiag, EAX

; Store cursor handles in handle window table:
	PUSH	ESI
	PUSH	EBX
	LEA	ESI, HandleInfo
	MOV	EAX, cursorLdiag
	MOV	EBX, cursorEW
	MOV	ECX, cursorRdiag
	MOV	EDX, cursorNS
; 1:
	MOV	[ESI + $handleInfo.cursor], EAX
; 2:
	MOV	[ESI + (SIZEOF $handleInfo) + $handleInfo.cursor], EDX
; 3:
	MOV	[ESI + (SIZEOF $handleInfo * 2) + $handleInfo.cursor], ECX
; 4:
	MOV	[ESI + (SIZEOF $handleInfo * 3) + $handleInfo.cursor], EBX
; 5:
	MOV	[ESI + (SIZEOF $handleInfo * 4) + $handleInfo.cursor], EBX
; 6:
	MOV	[ESI + (SIZEOF $handleInfo * 5) + $handleInfo.cursor], ECX
; 7:
	MOV	[ESI + (SIZEOF $handleInfo * 6) + $handleInfo.cursor], EDX
; 8:
	MOV	[ESI + (SIZEOF $handleInfo * 7) + $handleInfo.cursor], EAX

	POP	EBX
	POP	ESI
	RET

LoadArrowCursors	ENDP



FindHandleWindow	PROC

	MOV	ECX, 8
	LEA	EDX, HandleInfo
fhw10:	CMP	EAX, [EDX + $handleInfo.handle]	;Check handle for match.
	JE	fhw44				;Bingo! Return w/cursor.
	ADD	EDX, SIZEOF $handleInfo
	LOOP	fhw10
	XOR	EAX, EAX			;Fell through, no match.
	RET

fhw44:	MOV	EAX, [EDX + $handleInfo.cursor]
	RET

FindHandleWindow	ENDP



;====================================================================
; CenterDim
;
; Returns half screen dimension (X or Y) minus half window dimension
;
; On entry,
;	EAX = screen dimension
;	EDX = window dimension
;
; Returns:
;	sdim/2 - wdim/2
;====================================================================

CenterDim	PROC

	SHR	EAX, 1				;divide screen dimension by 2
	SHR	EDX, 1				;divide window dimension by 2
	SUB	EAX, EDX
	RET					;Return w/# in EAX

CenterDim	ENDP


	END	start



